"use strict";
this.name = "oolite-random-hits-mark";
this.author = "LittleBear";
this.copyright = "September 2008 - But do what you like with it in your OXPs!";
this.description	= "Tests who destroyed a RandomHits OXP Victim Ship and how they did it. The main script acts on the setting of variables.";
this.licence     = "CC BY-NC-SA 3.0";
this.version = "2.8";

//2.8 fixed mark naming when random shipnames oxp is not present. -spara-

/*
All the marks have a similar AI. The only difference is the random factor in decicions. So instead of maintaining 6 versions of
the AI it is easyer to just have one AI that calls the shipscript for making the desicions. The level of the target is defined
in the script_info of ships in  shipData.plist.
Level 1 marks had two different scripts. The 1s version had a higher tendency to beg for life than the 1 version and I gave it a level 0 value
for the differentiation between both. (it was only the boy-racer with the 1s script) The 2a and the 3a version had just no "fireMissile" command.
Therefor I introduced a "hasExtraMissileLauncher" variable. Now all ships call the scriptMessage to fire a missile, but only those that used
to fire missiles by AI will actually fire missiles. 1.74 will also allow missile manipulation so you could create a ship that always selects a
random type of the installed missiles.
*/

this.shipSpawned = function ()
{
    //this.ship.name = "Random Hits Mark"; // name only visible in debugging logs.
    if (!worldScripts.randomshipnames)
		this.ship.displayName = this.ship.name + ": " + expandDescription("[random_hits_main_name]");
	else
		this.ship.displayName = missionVariables.random_hits_store_mark_ship_name;
	// Show the Criminal's name on the ID Computer when targeted.
		
    this.random_hits_whoshot = "NO_ATTACK"; // start with assuming its no-attack.
    // this.ship.inspect(); // this line opens the target inspector on a mac with the debugConsole.oxp installed
    this.hasExtraMissileLauncher = this.ship.scriptInfo.extraMissileLauncher;
    if (this.hasExtraMissileLauncher === "no") this.hasExtraMissileLauncher = false; // test needed in case the code is in open-step plist
    this.level = parseInt(this.ship.scriptInfo.markLevel); // the level of the mark.
    this.ship.accuracy = this.level * ( 1 + 4 * missionVariables.random_hits_markposition / 100) - 6; // -4.6 till 7.8
    delete this.shipSpawned;
}

this.shipBeingAttacked = function (who)
{
    if (!who) return;
    if (who.isPlayer)
    {
        this.random_hits_whoshot = "PLAYER_ATTACK"; // Shooter was the player not using an e-bomb, q-mine or cloak. The variable remains undefined if the player e-bombs, q-mines or kills the victim whilst cloaked as thisBeingAttacked does not get run. The main script then awards the player the credit for the kill, but fines him for dishonerable conduct!
    }
    else
    {
        this.random_hits_whoshot = "NPC_ATTACK"; // Shooter was NPC ship. Every time the victim is hit, the variable is reset. Once he's dead the variable is not longer reset. So the last ship to hit him was also the killer!
        log(this.name, "Mark was hit by: "+who);
    }
};

this.shipDied = function (whom, cause)
{
    log(this.name, "Mark was killed by: " + whom + " Because of: " + cause + ". Last recorded hit: "+this.random_hits_whoshot);
    if (whom && whom.isPlayer)
    {
        if (this.random_hits_whoshot === "NPC_ATTACK" || cause === "cascade weapon")
        // Player has finished off a Mark attacked by an NPC or with an e-bomb or by q-bomb.
        // The Player is credited with the kill, but as he's a cowardly custard he is fined and he won't earn any points for completing this hit!
        {
            missionVariables.random_hits_status = "COMPLETE"; // Record the fact that the Victim's ship has been destroyed.
            worldScripts["Random_Hits"].$unmarkSystem(missionVariables.random_hits_planetnumber);
            player.credits -= missionVariables.random_hits_fine;
            mission.setInstructionsKey("random_hits_shortdescription2", "Random_Hits"); // Update the mission info.
            player.consoleMessage(expandDescription("[random_hits_complete_ebomb]"));
            missionVariables.random_hits_store_skill = 0;
        }
        else // The player did a honest kill...
        {
            if (Math.random() < parseFloat(this.ship.scriptInfo.podChance)) // Chance of Victim making it to his pod.
            {
                log(this.name, "Mark podded out of his ship");
                missionVariables.random_hits_status = "PODEDOUT"; // Set the variable to reflect that the player still needs to kill or scoop the Pod!
                this.ship.awardEquipment("EQ_ESCAPE_POD"); // Award an escape capsule
                this.ship.abandonShip(); // and bail out. Any shooting or scooping of the Pod is handled by the Pod's own Script.
                mission.setInstructionsKey("random_hits_shortdescription4", "Random_Hits"); // Update the mission info.
                player.consoleMessage(expandDescription("[random_hits_poded]"));
                return;

            }

            if (Math.random() < parseFloat(this.ship.scriptInfo.qmineChance)) // Chance of Victim dropping a q-mine.
            {	
                this.ship.spawnOne("RANDOM_HITS_MINE"); // Spawn a Q-Mine which taunts the player before exploding! Nasty!
            }
            missionVariables.random_hits_status = "COMPLETE"; // Record the fact that the Victim's ship has been destroyed.
            worldScripts["Random_Hits"].$unmarkSystem(missionVariables.random_hits_planetnumber);
            mission.setInstructionsKey("random_hits_shortdescription2", "Random_Hits"); // Update the mission info.
            player.consoleMessage(expandDescription("[random_hits_complete_comms]"));
        }
        this.setRevengeMark();
    }
    else // whom is not the player
    {
        if (this.random_hits_whoshot === "NPC_ATTACK") // If an NPC fired the fatal shot ...
        {	
            missionVariables.random_hits_deathcause = expandDescription("[random_hits_deathcause]");
            mission.setInstructionsKey("random_hits_shortdescription6", "Random_Hits"); // Update the mission info.
            player.consoleMessage(expandDescription("[random_hits_npckill]"));
        }
        else // Deals with any other situation where cause of death was neither the player nor an NPC Laser. EG: Victim did somthing silly like crashing, burnt up near the Sun or some other odd death!
        {	
            missionVariables.random_hits_deathcause = expandDescription("[random_hits_deathcause_accident]");
            mission.setInstructionsKey("random_hits_shortdescription6a", "Random_Hits"); // Update the mission info.
            player.consoleMessage(expandDescription("[random_hits_accident]"));
        }
        missionVariables.random_hits_status = "NPCKILL"; // Record the fact that the Victim's ship has been destroyed.
        missionVariables.random_hits_npcdays = clock.days;
        worldScripts["Random_Hits"].$unmarkSystem(missionVariables.random_hits_planetnumber);
    }
};

this.shipWillEnterWormhole = function ()
{
    log(this.name, "Mark has jumped out of the "+system.info.name+" system.");
    // If the mark jumped out of the system, set the mission description to tell the player to follow him.
    missionVariables.random_hits_status = "JUMPED";
    mission.setInstructionsKey("random_hits_shortdescription7", "Random_Hits"); // Update the mission info.
    player.consoleMessage(expandDescription("[random_hits_jumped_comms]"));
    worldScripts["Random_Hits"].$unmarkSystem(missionVariables.random_hits_planetnumber);
};

this.shipExitedWormhole = function ()
{
    log(this.name, "Mark has jumped into the "+system.info.name+" system.");
    // when the mark jumped enough to become clean, let him play the innocent trader. 
    if (this.ship.bounty === 0)
    {
        this.ship.switchAI("route1traderAI.plist");
        system.addShips("police", 1);
        system.addShips("hunter", 2);
    }
};

this.setRevengeMark = function(){
    missionVariables.random_hits_revenge_mark_first_name = missionVariables.random_hits_store_mark_first_name;
    missionVariables.random_hits_revenge_mark_second_name = missionVariables.random_hits_store_mark_second_name;
};

this.fireMissile = function (){
    // just give some ships the ability to fire missiles at will by the AI, not all.
    if (this.hasExtraMissileLauncher) this.ship.fireMissile();
};


/*
    below are chace functions called by AI, based on mark levels.
    Level 0 is a special case of level 1 that is only used for some ships but are part
    of the easy targets in the mission screens.
*/
this.reactToPlayer = function ()
{
    var chance = Math.random();
    switch(this.level)
    {
        case 0:
        case 1:
        {
            if (chance < 0.66) this.ship.reactToAIMessage("ON_COURSE")
            else if (chance < 0.88) this.ship.reactToAIMessage("FIGHT_PLAYER")
            else this.ship.reactToAIMessage("FLEE_PLAYER");
            break;
        }
        case 2:
        {
            if (chance < 0.11) this.ship.reactToAIMessage("ON_COURSE")
            else if (chance < 0.66) this.ship.reactToAIMessage("FIGHT_PLAYER")
            else this.ship.reactToAIMessage("FLEE_PLAYER");
            break;
        }
        case 3:
        {
            if (chance < 0.22) this.ship.reactToAIMessage("ON_COURSE")
            else this.ship.reactToAIMessage("FIGHT_PLAYER");
            break;
        }
        default:
        {
            this.ship.reactToAIMessage("FIGHT_PLAYER");
            break;
        }
    }
}

this.reactToPlayerAttack = function ()
{
    var chance = Math.random();
    switch(this.level)
    {
        case 0:
        case 1:
        {
            if (chance < 0.77) this.ship.reactToAIMessage("FIGHT_PLAYER")
            else this.ship.reactToAIMessage("FLEE_PLAYER");
            break;
        }
        case 2:
        case 3:
        {
            if (chance < 0.88) this.ship.reactToAIMessage("FIGHT_PLAYER")
            else this.ship.reactToAIMessage("FLEE_PLAYER");
            break;
        }
        default:
        {
            this.ship.reactToAIMessage("FIGHT_PLAYER");
            break;
        }
    }
}

this.markHurt = function ()
{
    var chance = Math.random();
    switch(this.level)
    {
        case 0:
        {
            if (chance < 0.37) this.ship.reactToAIMessage("BEG_FOR_LIFE")
            else if (chance < 0.59) this.ship.reactToAIMessage("FLEE_PLAYER")
            else if (chance < 0.58) this.ship.reactToAIMessage("FIGHT_PLAYER")
            else this.ship.reactToAIMessage("JUMP_OUT");
            break;
        }
        case 1:
        {
            if (chance < 0.33) this.ship.reactToAIMessage("BEG_FOR_LIFE")
            else if (chance < 0.55) this.ship.reactToAIMessage("FLEE_PLAYER")
            else if (chance < 0.77) this.ship.reactToAIMessage("FIGHT_PLAYER")
            else this.ship.reactToAIMessage("JUMP_OUT");
            break;
        }
        case 2:
        {
            if (chance < 0.11) this.ship.reactToAIMessage("BEG_FOR_LIFE")
            else if (chance < 0.33) this.ship.reactToAIMessage("FLEE_PLAYER")
            else if (chance < 0.66) this.ship.reactToAIMessage("FIGHT_PLAYER")
            else this.ship.reactToAIMessage("JUMP_OUT");
            break;
        }
        case 3:
        {
            if (chance < 0.11) this.ship.reactToAIMessage("BEG_FOR_LIFE")
            else if (chance < 0.66) this.ship.reactToAIMessage("FIGHT_PLAYER")
            else this.ship.reactToAIMessage("JUMP_OUT");
            break;
        }
        default:
        {
            this.ship.reactToAIMessage("FIGHT_PLAYER");
            break;
        }
    }
}

this.fleeMine = function ()
{
    var chance = Math.random();
    switch(this.level)
    {
        case 0:
        case 1:
        {
            if (chance < 0.5) this.ship.reactToAIMessage("JUMP_OUT")
            else this.ship.reactToAIMessage("MINE_RUN");
            break;
        }
        case 2:
        case 3:
        {
            if (chance < 0.66) this.ship.reactToAIMessage("JUMP_OUT")
            else this.ship.reactToAIMessage("MINE_RUN");
            break;
        }
        default:
        {
            this.ship.reactToAIMessage("JUMP_OUT");
            break;
        }
    }
}
